
#include <inttypes.h>
#include <compat/twi.h>

/* define CPU frequency in Mhz here if not defined in Makefile */
#ifndef F_CPU
#define F_CPU 11059000L
#endif

/* I2C clock in Hz */
#define SCL_CLOCK  250000L


/*************************************************************************/

#define Dev_AT24C512  0b10101000     // device address of EEPROM, see datasheet

// defines the data direction (reading from I2C device) in i2c_start()
#define I2C_READ    1

// defines the data direction (writing to I2C device) in i2c_start()
#define I2C_WRITE   0

/*************************************************************************
 Initialization of the I2C bus interface. Need to be called only once
*************************************************************************/
void i2c_init(void)
{
  /* SCL_CLOCK=(F_CPU/(16+(2*TWBR)*4^TWPS))
  TWBR = Value of the TWI Bit Rate Register
  TWPS = Value of the prescaler bits in the TWI Status Register
  */
  
  /* initialize TWI clock: 250 kHz clock, TWPS = 0 => prescaler = 1 */
  
  TWSR = 0;                         /* no prescaler */
  TWBR = ((F_CPU/SCL_CLOCK)-16)/2;  /* must be > 10 for stable operation */

}


/*************************************************************************	
  Issues a start condition and sends address and transfer direction.
  return 0 = device accessible, 1= failed to access device
*************************************************************************/
unsigned char i2c_start(void)
{
    uint8_t   twst;

	// send START condition
	TWCR = (1<<TWINT) | (1<<TWSTA) | (1<<TWEN);

	//TWINT: TWI Interrupt Flag  The TWINT Flag must be cleared by software by writing  
	//a logic one to it.
	//TWSTA: TWI START Condition Bit
	//TWEN: TWI Enable Bit

	// wait until transmission completed
	while(!(TWCR & (1<<TWINT)));

	//wait for TWINT Flag set, indicates that the START condition has been transmitted
	//TWINT: This bit is set by hardware when the TWI has finished its current job and 
	//expects application software response.

	// check value of TWI Status Register. Mask prescaler bits.
	twst = TWSR & 0xF8;
	if ( (twst != TW_START) && (twst != TW_REP_START) ) return 1;	

	//Status Code
	//TW_START 		= 0x08	A START condition has been transmitted
	//TW_REP_START 	= 0x10	A repeated START condition has been transmitted
	//return 1 = Error

	return 0;
}

unsigned char i2c_device_address(unsigned char address){
    uint8_t   twst;
	// send device address
	TWDR = address;
	TWCR = (1<<TWINT) | (1<<TWEN);

	// wail until transmission completed and ACK/NACK has been received
	while(!(TWCR & (1<<TWINT)));

	// check value of TWI Status Register. Mask prescaler bits.
	twst = TWSR & 0xF8;
	if ( (twst != TW_MT_SLA_ACK) && (twst != TW_MR_SLA_ACK) ) return 1;

	//Status Code
	//TW_MT_SLA_ACK = 0x18	SLA+W has been transmitted; ACK has been received
	//TW_MR_SLA_ACK = 0x40	SLA+R has been transmitted; ACK has been received
	//return 1 = no Ack

	return 0;
}


/*************************************************************************
 Terminates the data transfer and releases the I2C bus
*************************************************************************/
void i2c_stop(void)
{
    /* send stop condition */
	TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO);
	
	// wait until stop condition is executed and bus released
	while(TWCR & (1<<TWSTO));

	/*Writing the TWSTO bit to one in Master mode will generate a STOP 
	condition on the Two-wire Serial Bus. When the STOP condition is 
	executed on the bus, the TWSTO bit is cleared automatically.*/
}


/*************************************************************************
  Send one byte to I2C device
  
  Input:    byte to be transfered
  Return:   0 write successful 
            1 write failed
*************************************************************************/
unsigned char i2c_write( unsigned char data )
{	
    uint8_t   twst;
    
	// send data to the previously addressed device
	TWDR = data;
	TWCR = (1<<TWINT) | (1<<TWEN);

	// wait until transmission completed
	while(!(TWCR & (1<<TWINT)));

	// check value of TWI Status Register. Mask prescaler bits
	twst = TWSR & 0xF8;
	if( twst != TW_MT_DATA_ACK) return 1;

	//Status Code
	//TW_MT_DATA_ACK = 0x28	Data byte has been transmitted; ACK has been received
	//return 1 = no Ack, transmission of data byte failed

	return 0;
}


/*************************************************************************
 Read one byte from the I2C device, request more data from device 
 
 Return:  byte read from I2C device
*************************************************************************/
unsigned char i2c_read_Ack(void)
{
	TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWEA);

	/*TWEA: TWI Enable Acknowledge Bit
	If the TWEA bit is written to one, the ACK pulse is generated on the TWI bus 
	if [...] a data byte has been received in Master Receiver [...] mode. */

	// wait until transmission completed
	while(!(TWCR & (1<<TWINT)));  	  
    return TWDR;

/*  sequential read:
	After C receives a data byte, it responds with an acknowledge. As long as
	EEPROM receives an acknowledge, it will continue to serially clock out
	sequential data bytes.
	The read operation is terminated with no acknowledge and stop condition.
*/
}


/*************************************************************************
 Read one byte from the I2C device, read is followed by a stop condition 
 
 Return:  byte read from I2C device
*************************************************************************/
unsigned char i2c_read_Nack(void)
{
	TWCR = (1<<TWINT) | (1<<TWEN);
	while(!(TWCR & (1<<TWINT)));
    return TWDR;
}





